@@ -4,7 +4,7 @@ from pobm._ErrorHandler import _check_shape_, WrongParameter
 import numpy as np
 import warnings
 
-from pobm._ResultsClasses import DesaturationsMeasuresResults
+from pobm._ResultsClasses import DesaturationsMeasuresResults, DesatMethodEnum
 
 
 class DesaturationsMeasures:
@@ -12,16 +12,21 @@ class DesaturationsMeasures:
     Class that calculates the desaturation features from SpO2 time series.
     """
 
-    def __init__(self, ODI_Threshold: int = 3, hard_threshold: int = 90, relative: bool = True,
+    def __init__(self, ODI_Threshold: int = 3, hard_threshold: int = 90,
+                 threshold_method: DesatMethodEnum = DesatMethodEnum.Relative, quantile_threshold: float = 0.10,
                  desat_max_length: int = 90, min_dist_meta_event: int = 0, desat_min_length: int = 0):
         """
 
-        :param ODI_Threshold: Threshold to compute Oxygen Desaturation Index.
+        :param threshold_method: Select the threshold method to detect desaturations
+        :type threshold_method: DesatMethodEnum, optional
+
+        :param ODI_Threshold: Threshold to compute Oxygen Desaturation Index. Used only if threshold_method = DesatMethodEnum.Relative
         :type ODI_Threshold: int, optional
-        :param hard_threshold: Hard threshold to detect desaturations.
+        :param hard_threshold: Hard threshold to detect desaturations. Used only if threshold_method = DesatMethodEnum.Hard
         :type hard_threshold: int, optional
-        :param relative: Whether to use a relative or hard threshold to detect desaturations.
-        :type relative: bool, optional
+        :param quantile_threshold: Threshold to detect desaturations. Used only if threshold_method = DesatMethodEnum.Quantile
+        :type quantile_threshold: float, optional
+
         :param desat_max_length: The maximum length of desaturations.
         :type desat_max_length: int, optional
         :param min_dist_meta_event: The maximal distance between 2 desaturations to convert them into a meta desat
@@ -35,9 +40,12 @@ class DesaturationsMeasures:
         if desat_max_length <= 0:
             raise WrongParameter("desat_max_length should be strictly positive")
 
+        self.threshold_method = threshold_method
+
         self.ODI_Threshold = ODI_Threshold
         self.hard_threshold = hard_threshold
-        self.relative = relative
+        self.quantile_threshold = quantile_threshold
+
         self.desat_max_length = desat_max_length
         self.min_dist_meta_event = min_dist_meta_event
         self.desat_min_length = desat_min_length
@@ -46,11 +54,16 @@ class DesaturationsMeasures:
         self.end = []
         self.counter_desat = []
 
-    def compute(self, signal) -> DesaturationsMeasuresResults:
+    def compute(self, signal: np.ndarray, begin: np.ndarray = None,
+                end: np.ndarray = None) -> DesaturationsMeasuresResults:
         """
         Computes all the biomarkers of this category.
 
         :param signal: 1-d array, of shape (N,) where N is the length of the signal
+        :param begin: Numpy array of indices of beginning of each desaturation event. Default value is None.
+        :type begin: Numpy array, optional
+        :param end: Numpy array of indices of end of each desaturation event. begin and end should have the same length. Default value is None.
+        :type end: Numpy array, optional
         :return: DesaturationsMeasuresResults class containing the following features:
 
             * ODI: Oxygen Desaturation Index
@@ -76,26 +89,69 @@ class DesaturationsMeasures:
         .. code-block:: python
 
             from pobm.obm.desat import DesaturationsMeasures
+            from pobm._ResultsClasses import DesatMethodEnum
 
             # Initialize the class with the desired parameters
-            desat_class = DesaturationsMeasures(ODI_Threshold=3)
+            desat_class = DesaturationsMeasures(ODI_Threshold=3, threshold_method=DesatMethodEnum.Relative)
 
             # Compute the biomarkers
             results_desat = desat_class.compute(spo2_signal)
 
+            # Compute the biomarkers with known desaturation locations
+            results_desat = desat_class.compute(spo2_signal, begin, end)
+
         """
+        # Initial data checks
         _check_shape_(signal)
 
         warnings.simplefilter('ignore', np.RankWarning)
         warnings.filterwarnings("ignore", category=RuntimeWarning)
 
-        if self.relative is True:
-            self.desaturation_detector(signal)
-        else:
-            self.__hard_threshold_detector(signal)
+        if (begin is None) or (end is None):
+            if self.threshold_method == DesatMethodEnum.Relative:
+                self.__desaturation_detector(signal)
+            elif (self.threshold_method == DesatMethodEnum.Hard) or (self.threshold_method == DesatMethodEnum.Quantile):
+                self.__hard_threshold_detector(signal)
 
-        self.group_meta_desat(signal)
-        self.remove_small_desats()
+            self.__group_meta_desat(signal)
+        else:
+            if isinstance(begin, int):
+                begin = np.array([begin])
+            if isinstance(end, int):
+                end = np.array([end])
+
+            if len(begin) != len(end):
+                raise WrongParameter("The parameters begin and end should have the same length")
+
+            assert all(begin > 0) and all(end > 0) and all((end - begin) > 0), \
+                "All events indexes should be strictly positive and end values should be after begin values"
+            assert end[-1] < signal.shape[0], "Last annotated event should not cross the end of the signal"
+
+            # Sort arrays according to begin
+            s_idx = np.argsort(begin)
+            begin = begin[s_idx]
+            end = end[s_idx]
+
+            # Move begin and end to class attirbutes
+            self.begin = begin
+            self.end = end
+            self.min_desat = [x + np.argmin(signal[x:y]) for (x, y) in zip(begin, end)]
+            self.counter_desat = []
+
+        self.__remove_small_desats()
+
+        return self.__get_desaturation_features(signal)
+
+    def __get_desaturation_features(self, signal) -> DesaturationsMeasuresResults:
+        """
+        Computes all the biomarkers of this category from pre-loaded signal and desaturation intervals (see compute functions).
+
+        :param signal: 1-d array, of shape (N,) where N is the length of the signal
+
+        """
+        # Check if begin and end are empty to prevent errors when calling this function directly
+        if len(self.begin) == 0 or len(self.end) == 0:
+            return DesaturationsMeasuresResults(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, [], [])
 
         ODI = len(self.begin) / len(signal) * 3600
 
@@ -191,7 +247,7 @@ class DesaturationsMeasures:
 
         return desaturation_features
 
-    def group_meta_desat(self, signal):
+    def __group_meta_desat(self, signal):
         end_before = - self.min_dist_meta_event - 5
 
         new_idx_desat = np.zeros(shape=(len(self.begin)), dtype=np.int8)
@@ -228,7 +284,7 @@ class DesaturationsMeasures:
         counter_desat = Counter(new_idx_desat)
         self.counter_desat = list(counter_desat.values())
 
-    def remove_small_desats(self):
+    def __remove_small_desats(self):
         new_begin, new_end, new_min, new_counter = [], [], [], []
         for i_begin, i_end, i_min, i_counter in zip(self.begin, self.end, self.min_desat, self.counter_desat):
             if i_end - i_begin > self.desat_min_length:
@@ -242,7 +298,7 @@ class DesaturationsMeasures:
         self.min_desat = new_min
         self.counter_desat = new_counter
 
-    def desaturation_detector(self, signal):
+    def __desaturation_detector(self, signal):
         """
         Runs desaturation detector, compute the ODI biomarker according to [6]_
 
@@ -391,14 +447,19 @@ class DesaturationsMeasures:
         :param signal: The SpO2 signal, of shape (N,)
         :return: ODI
         """
-        hard_threshold = self.hard_threshold
+
+        if self.threshold_method == DesatMethodEnum.Hard:
+            hard_threshold = self.hard_threshold
+        else:
+            hard_threshold = np.quantile(signal, self.quantile_threshold)
+
         begin_desat, min_desat, end_desat = [], [], []
         turn_begin = True
 
         for i in range(len(signal)):
             if i == 0:
                 continue
-            if signal[i] != signal[i]:  # Check for NaN
+            if np.isnan(signal[i]):  # Check for NaN
                 continue
             if (signal[i - 1] >= hard_threshold) and (signal[i] < hard_threshold):
                 if turn_begin is True:
